package test;

import java.io.File;
import java.net.URL;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class Main {

   private static final Log LOG = LogFactory.getLog(Main.class);

   private static final String PID_FILENAME_KEY = "test.pidfile";
   private static final String PID_FILENAME_DEFAULT = "test.pid";

   private static Thread mainThread;

   private volatile boolean cancled = false;
   private Thread listener;

   private static Thread getMainDaemonThread() {
      return mainThread;
   }

   static private void daemonize() {
      mainThread = Thread.currentThread();
      getPidFile().deleteOnExit();
   }

   private static File getPidFile() {
      return new File(System.getProperty(PID_FILENAME_KEY, PID_FILENAME_DEFAULT));
   }

   private void startCancleListener() {
      this.listener = new Thread() {
         @Override
         public void run() {
            LOG.warn("Invoke stopping...");
            Main.this.cancled = true;

            try {
               getMainDaemonThread().join();
            } catch (final InterruptedException e) {
               LOG.error("Interrupted which waiting on main daemon thread to complete.");
            }

         }

      };

      if (LOG.isInfoEnabled()) {
         LOG.info("Add shutdown hook...");
      }
      this.listener.setDaemon(true);
      Runtime.getRuntime().addShutdownHook(this.listener);
   }

   private void interruptCancleListener() {
      if (LOG.isInfoEnabled()) {
         LOG.info("Interrupting cancle listner...");
      }
      Runtime.getRuntime().removeShutdownHook(this.listener);
      this.listener.interrupt();
   }

   public static void main(String[] args) throws Exception {

      try {
         // do sanity checks and startup actions
         daemonize();
      } catch (final Throwable e) {
         LOG.fatal("Startup failed", e);
      }

      final Main main = new Main();
      main.startCancleListener();
      try {
         main.run();
      } catch (final RuntimeException e) {
         LOG.error("Uncaught and unhandled error occurs. Please report this bug", e);
         main.interruptCancleListener();
         System.exit(1);
      }
      LOG.info("Main stops");
      System.exit(0);

   }

   private void run() throws Exception {
      final int maxParallel = 200;
      final URL wsdlUrl = new URL("http://localhost:8080/MyWsBeanService/MyWsBean?wsdl");
      final QName qName = new QName("http://test/", "MyWsBeanService");

      final Service service = Service.create(wsdlUrl, qName);

      final ExecutorService threadPool = new ThreadPoolExecutor(maxParallel, maxParallel, 1L, TimeUnit.DAYS,
               new LinkedBlockingQueue<Runnable>());

      long counter = 0;
      while (!this.cancled) {
         try {
            counter++;
            final MyWs myWs = service.getPort(MyWs.class);
            threadPool.execute(new WsCall(myWs, counter));
         } catch (Exception e) {
            LOG.warn("Failed to ws call with value " + counter, e);
         }
      }
      LOG.info("Invoke threads shutdown...");
      threadPool.shutdown();
      try {
         threadPool.awaitTermination(10000, TimeUnit.NANOSECONDS);
      } catch (final InterruptedException e) {
         LOG.warn("failed to wait for ending of all threads", e);
      }
      threadPool.shutdownNow();
   }

   private static class WsCall implements Runnable {

      private final MyWs myWs;
      private final long value;

      public WsCall(final MyWs myWs, final long value) {
         this.myWs = myWs;
         this.value = value;
      }

      @Override
      public void run() {
         try {
            LOG.info("WsCall wiht value " + value);
            myWs.callMe(value);
         } catch (Exception e) {
            LOG.error("WsCall with value " + value + " failed", e);
         }
      }

   }
}
